Package: spacetime
Version: 1.3-2
Title: Classes and Methods for Spatio-Temporal Data
Authors@R: c(person("Edzer",  "Pebesma", role = c("aut", "cre"), email = "edzer.pebesma@uni-muenster.de", comment = c(ORCID = "0000-0001-8049-7069")),
	person("Benedikt", "Graeler", role = "ctb"),
	person("Tom", "Gottfried", role = "ctb"),
	person("Robert J.", "Hijmans", role = "ctb"))
Depends: R (>= 3.0.0)
Imports: graphics, utils, stats, methods, lattice, sp (>= 1.1-0), zoo
        (>= 1.7-9), xts (>= 0.8-8), intervals
Suggests: adehabitatLT, cshapes (>= 2.0), foreign, gstat (>= 1.0-16),
        maps, mapdata, plm, raster, RColorBrewer, rmarkdown,
        RPostgreSQL, knitr, ISOcodes, markdown, sf, sftime
LazyData: no
Description: Classes and methods for spatio-temporal data, including space-time regular lattices, sparse lattices, irregular data, and trajectories; utility functions for plotting data as map sequences (lattice or animation) or multiple time series; methods for spatial and temporal selection and subsetting, as well as for spatial/temporal/spatio-temporal matching or aggregation, retrieving coordinates, print, summary, etc.
License: GPL (>= 2)
URL: https://github.com/edzer/spacetime
BugReports: https://github.com/edzer/spacetime/issues
Encoding: UTF-8
VignetteBuilder: knitr
Collate: Class-xts.R Class-ST.R Class-STFDF.R Class-STSDF.R
        Class-STIDF.R Class-STTDF.R interval.R endtime.R ST-methods.R
        STFDF-methods.R STSDF-methods.R STIDF-methods.R STTDF-methods.R
        apply.R coerce.R stconstruct.R plot.R stplot.R timematch.R
        over.R aggregate.R eof.R mnf.R bind.R na.R raster.R tgrass.R
        stinteraction.R
NeedsCompilation: no
Packaged: 2024-09-04 19:53:57 UTC; edzer
Author: Edzer Pebesma [aut, cre] (<https://orcid.org/0000-0001-8049-7069>),
  Benedikt Graeler [ctb],
  Tom Gottfried [ctb],
  Robert J. Hijmans [ctb]
Maintainer: Edzer Pebesma <edzer.pebesma@uni-muenster.de>
Repository: CRAN
Date/Publication: 2024-09-04 23:00:02 UTC
Built: R 4.4.1; ; 2024-09-06 13:59:08 UTC; unix
